@@ -14,6 +14,7 @@ extern void flush_redo_stack();
 extern void truncate_undo_stack();
 extern void free_undo_record(Undoptr);
 extern void free_redo_record(Undoptr);
+extern stringpart *get_original_string(labelptr);
 #ifndef TCL_WRAPPER
 extern void undo_call(xcWidget, caddr_t, caddr_t);
 extern void redo_call(xcWidget, caddr_t, caddr_t);
@@ -116,6 +117,7 @@ extern void endvbar(xcWidget, caddr_t, XButtonEvent *);
 extern void zoombox(xcWidget, caddr_t, caddr_t);
 extern void zoomin(xcWidget, caddr_t, XButtonEvent *);
 extern void zoomout(xcWidget, caddr_t, XButtonEvent *);
+extern void warppointer(int, int);
 extern void panbutton(u_int, XButtonEvent *);
 extern void zoominrefresh(xcWidget, caddr_t, XButtonEvent *);
 extern void zoomoutrefresh(xcWidget, caddr_t, XButtonEvent *);
@@ -175,6 +177,7 @@ extern void selectsave(xcWidget, caddr_t, caddr_t);
 extern void arceditpush(arcptr);
 extern void splineeditpush(splineptr);
 extern void polyeditpush(polyptr);
+extern pointlist copypoints(pointlist, int);
 extern void labelcopy(labelptr, labelptr);
 extern void arccopy(arcptr, arcptr);
 extern void polycopy(polyptr, polyptr);
@@ -214,7 +217,7 @@ extern void importspice();
 #endif
 
 extern char *ridnewline(char *);
-extern void free_single(genericptr *);
+extern void free_single(genericptr);
 extern void reset(objectptr, short);
 extern void pagereset(short);
 extern void initmem(objectptr);
@@ -304,6 +307,7 @@ extern void findsplinepos(splineptr, float, XPoint *, int *);
 extern void ffindsplinepos(splineptr, float, XfPoint *);
 extern float findsplinemin(splineptr, XPoint *);
 extern short closepoint(polyptr, XPoint *);
+extern short closedistance(polyptr, XPoint *);
 extern void updateinstparam(objectptr);
 extern short checkbounds();
 extern void window_to_user(short, short, XPoint *);
@@ -334,7 +338,7 @@ extern void InvTransformPoints(XPoint *, XPoint *, short, XPoint, float,
 extern void manhattanize(XPoint *, polyptr);
 extern void bboxcalc(short, short *, short *);
 extern void calcextents(genericptr *, short *, short *, short *, short *);
-extern void objinstbbox(objinstptr, XPoint *);
+extern void objinstbbox(objinstptr, XPoint *, Boolean);
 extern void labelbbox(labelptr, XPoint *, objinstptr);
 extern void calcbboxsingle(genericptr *, objinstptr, short *, short *, short *, short *);
 extern Boolean object_in_library(short, objectptr);
