@@ -33,12 +33,12 @@
 /*----------------------------------------------------------------------*/
 #include "prototypes.h"
 
-/*-------------------------------------------------------------------------*/
-/* Global Variable definitions						   */
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Exported Variable definitions					*/
+/*----------------------------------------------------------------------*/
 
 extern short eventmode;	/* keep track of the mode the screen is in */
-extern Display	*dpy;   /* Works well to make this globally accessible */
+extern Display	*dpy;
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern Clientdata areastruct;
@@ -232,7 +232,10 @@ int test_insideness(int tx, int ty, XPoint *boxpoints)
 /* Search for selection among path components */
 /*--------------------------------------------*/
 
-Boolean pathselect(genericptr *curgen, short class)
+#define RANGE_NARROW 11.5
+#define RANGE_WIDE 50
+
+Boolean pathselect(genericptr *curgen, short class, float range)
 {
    /*----------------------------------------------------------------------*/
    /* wirelim is the distance, in user-space units, at which an element is */
@@ -246,13 +249,13 @@ Boolean pathselect(genericptr *curgen, short class)
    /*		is an empirical result.					   */
    /*----------------------------------------------------------------------*/
 
-   float	wirelim = 2 + 11.5 / (*areastruct.vscale + 0.05);
+   float	wirelim = 2 + range / (*areastruct.vscale + 0.05);
    long		sqrwirelim = (int)(wirelim * wirelim);
 
    long		newdist;
    Boolean	selected = False;
 
-   class &= (areastruct.filter | SELECT_HIDE);	/* apply the selection filter */
+   class &= areastruct.filter;	/* apply the selection filter */
 
    if ((*curgen)->type == (class & ARC)) {
 
@@ -335,26 +338,18 @@ Boolean pathselect(genericptr *curgen, short class)
 /* Select one of the elements on-screen */
 /*--------------------------------------*/
 
-short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr selinst)
+selection *genselectelement(short class, u_char mode, objectptr selobj,
+		objinstptr selinst)
 {
-   short	*newselect, *foundtmp, *foundlist;
-
+   selection	*rselect = NULL;
+   short	*newselect;
    genericptr	*curgen;
    XPoint 	newboxpts[4];
-   XEvent 	event;
-   XButtonEvent *buttonevent = (XButtonEvent *)(&event);
-   Boolean	selected, unselect = False;
-   short	found = 0;
+   Boolean	selected;
+   float	range = RANGE_NARROW;
 
-   XDefineCursor(dpy, areastruct.areawin, QUESTION);
-
-   /* Definition for unselecting an element */
-
-   if (class < 0) {
-      unselect = True;
-      class = -class;
-   }
-   class &= (areastruct.filter | SELECT_HIDE);	/* apply the selection filter */
+   if (mode == MODE_RECURSE_WIDE)
+      range = RANGE_WIDE;
 
    /* Loop through all elements found underneath the cursor */
 
@@ -368,21 +363,12 @@ short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr s
 
       selected = False;
 
-      /* check for position inside the bounding box of one of the objects */
+      /* Check among polygons, arcs, and curves */
 
-      if ((*curgen)->type == (class & OBJINST)) {
-
-	 objinstbbox(TOOBJINST(curgen), newboxpts);
-
-         /* Look for an intersect of the boundingbox and pointer position. */
-         /* This is a simple matter of rotating the pointer position with  */
-         /*  respect to the origin of the bounding box segment, as if the  */
-         /*  segment were rotated to 0 degrees.  The sign of the resulting */
-         /*  point's y-position is the same for all bbox segments if the   */
-         /*  pointer position is inside the bounding box.		   */
-
-	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
-		newboxpts);
+      if (((*curgen)->type == (class & POLYGON)) ||
+		((*curgen)->type == (class & ARC)) ||
+		((*curgen)->type == (class & SPLINE))) {
+	  selected = pathselect(curgen, class, range);
       }
 
       else if ((*curgen)->type == (class & LABEL)) {
@@ -415,158 +401,48 @@ short *genselectelement(short class, u_char mode, objectptr selobj, objinstptr s
 
  	 for (pathp = TOPATH(curgen)->plist; pathp < TOPATH(curgen)->plist +
 		TOPATH(curgen)->parts; pathp++)
-	    if (pathselect(pathp, SPLINE|ARC|POLYGON)) {
+	    if (pathselect(pathp, SPLINE|ARC|POLYGON, range)) {
 	       selected = True;
 	       areastruct.editsubpart = (short)(pathp - TOPATH(curgen)->plist);
 	       break;
 	    }
       }
 
-      /* Check among polygons, arcs, and curves */
+      else if ((*curgen)->type == (class & OBJINST)) {
 
-      else selected = pathselect(curgen, class);
+	 objinstbbox(TOOBJINST(curgen), newboxpts, True);
 
-      /* check if this object has already been selected */
+         /* Look for an intersect of the boundingbox and pointer position. */
+         /* This is a simple matter of rotating the pointer position with  */
+         /*  respect to the origin of the bounding box segment, as if the  */
+         /*  segment were rotated to 0 degrees.  The sign of the resulting */
+         /*  point's y-position is the same for all bbox segments if the   */
+         /*  pointer position is inside the bounding box.		   */
 
-      for (newselect = areastruct.selectlist; newselect <
-             areastruct.selectlist + areastruct.selects; newselect++) 
-         if (*newselect == (short)(curgen - selobj->plist)) {
-	    if (!unselect) selected = False;
-	    break;
-	 }
-
-      if (unselect && (newselect == areastruct.selectlist + areastruct.selects))
-	 selected = False;
+	 selected = test_insideness(areastruct.save.x, areastruct.save.y,
+		newboxpts);
+      }
 
       /* Add this object to the list of things found under the cursor */
 
-      if (selected && (!unselect)) {
-
-         if (found == 0)
-            foundlist = (short *) malloc(sizeof(short));
-         else
-            foundlist = (short *) realloc(foundlist, (found + 1) * sizeof(short));
-
-         newselect = foundlist + found;
-         found++;
-     
-         if (selected) *newselect = (short)(curgen - selobj->plist);
+      if (selected) {
+         if (rselect == NULL) {
+	    rselect = (selection *)malloc(sizeof(selection));
+	    rselect->selectlist = (short *)malloc(sizeof(short));
+	    rselect->selects = 0;
+	    rselect->thisinst = selinst;
+	    rselect->next = NULL;
+	 }
+         else {
+            rselect->selectlist = (short *)realloc(rselect->selectlist,
+			(rselect->selects + 1) * sizeof(short));
+	 }
+	 *(rselect->selectlist + rselect->selects) = (short)(curgen -
+		selobj->plist);
+	 rselect->selects++;
       }
-
-      /* mechanism for unselecting elements under the cursor */
-
-      else if (selected) {
-         short *desel;
-
-         XSetFunction(dpy, areastruct.gc, GXcopy);
-         XTopSetForeground(GSELTOCOLOR(selobj, newselect));
-         geneasydraw(*newselect, DEFAULTCOLOR, selobj, selinst);
-         areastruct.selects--;
-         for (desel = newselect; desel < areastruct.selectlist +
-	        areastruct.selects; desel++)
-	    *desel = *(desel + 1);
-         if (areastruct.selects == 0) free (areastruct.selectlist);
-      }
-      else newselect = NULL;
-
-   } /* for-loop over all parts */
-
-   /* Various options depending on how many things were found */
-
-   switch (found) {
-      case 0: 
-	 newselect = NULL;
-	 break;
-
-      case 1: 
-	 newselect = allocselect();
-	 *newselect = *foundlist;
-	 free(foundlist);
-	 gendrawselected(newselect, selobj, selinst);
-	 break;
-
-      default: {
-	 newselect = NULL;
-	 XSetFunction(dpy, areastruct.gc, GXcopy);
-	 for (foundtmp = foundlist; foundtmp < foundlist + found; foundtmp++) {
-            XSetForeground(dpy, areastruct.gc, QUERYCOLOR);
-            geneasydraw(*foundtmp, DOFORALL, selobj, selinst);
-	    sprintf(_STR, "Click to accept/reject: %d of %d",
-		(int)(foundtmp - foundlist + 1), found);
-	    Wprintf(_STR);
-
-	    for(;;) {  	/* This flushes the X event queue */
-	       XNextEvent(dpy, &event); 
-	       if (event.type != ButtonPress && event.type != ButtonRelease
-		    && event.type != MotionNotify && event.type != KeyPress)
-	          xcDispatchEvent(&event);
-	       else if (event.type == ButtonPress) break;
-	    }
-	    if (buttonevent->button == Button3) {
-               XTopSetForeground(GSELTOCOLOR(selobj, foundtmp));
-               geneasydraw(*foundtmp, DEFAULTCOLOR, selobj, selinst);
-	    }
-	    else {
-	       newselect = allocselect();
-               *newselect = *foundtmp;
-               XSetForeground(dpy, areastruct.gc, SELECTCOLOR);
-               geneasydraw(*newselect, DOFORALL, selobj, selinst);
-	    }
-         }
-      }  break;
    }
-
-   if (found > 1) {
-      short *nselect;
-
-      XDefineCursor(dpy, areastruct.areawin, CIRCLE);
-      if (eventmode == PRESS_MODE)
-	 Wprintf("Click and hold button 1 to continue move.");
-      else
-	 Wprintf("Selection complete");
-
-      for(;;) {
-	 XNextEvent(dpy, &event); 
-	 if (event.type != ButtonPress && event.type != ButtonRelease &&
-		event.type != MotionNotify && event.type != KeyPress)
-	    xcDispatchEvent(&event);
-	 else if ((event.type == ButtonPress) && (eventmode == PRESS_MODE))
-	    break;
-	 else if ((event.type == ButtonRelease) && (eventmode != PRESS_MODE))
-	    break;
-      }
-      free(foundlist);
-
-      /* make it work right for smooth xor'ing */
-
-      XSetFunction(dpy, areastruct.gc, GXcopy);
-      XSetForeground(dpy, areastruct.gc, BACKGROUND);
-      for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
-	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj, selinst);
-      XSetFunction(dpy, areastruct.gc, GXxor);
-      XSetForeground(dpy, areastruct.gc, SELECTCOLOR ^ BACKGROUND);
-      for (nselect = areastruct.selectlist; nselect < areastruct.selectlist
-	   + areastruct.selects; nselect++)
-         geneasydraw(*nselect, DOFORALL, selobj, selinst);
-   }
-
-   setoptionmenu();
-
-   if (eventmode == DELETE_MODE)
-      XDefineCursor(dpy, areastruct.areawin, SCISSORS);
-   else
-      XDefineCursor(dpy, areastruct.areawin, CROSS);
-
-   /* snap the coordinates */
-
-   u2u_snap(&areastruct.save);
-
-   /* Register the selection as an undo event */
-   register_for_undo(XCF_Select, mode, areastruct.topinstance,
-		areastruct.selectlist, areastruct.selects);
-
-   return newselect;
+   return rselect;
 }
 
 /*----------------------------------------------------------------*/
@@ -652,7 +528,7 @@ void selectarea()
 	+ topobject->parts; curgen++) {
 
       /* apply the selection filter */
-      if (!((*curgen)->type & (areastruct.filter | SELECT_HIDE))) continue;
+      if (!((*curgen)->type & areastruct.filter)) continue;
 
       switch(ELEMENTTYPE(*curgen)) {
 	case(OBJINST):
@@ -831,6 +707,23 @@ void draw_normal_selected(objectptr thisobj, objinstptr thisinst)
    }
 }
 
+/*----------------------------------------------------------------------*/
+/* Free a selection linked-list structure				*/
+/* (don't confuse with freeselects)					*/
+/*----------------------------------------------------------------------*/
+
+static void freeselection(selection *rselect)
+{
+   selection *nextselect;
+
+   while (rselect != NULL) {
+      nextselect = rselect->next;
+      free(rselect->selectlist);
+      free(rselect);
+      rselect = nextselect;
+   }
+}
+
 /*--------------------------------------------------------------*/
 /* Free memory from the previous selection list, copy the	*/
 /* current selection list to the previous selection list, and	*/
@@ -876,7 +769,7 @@ void clearselects()
 }
 
 /*--------------------------------------------------------------*/
-/* Unselect all the selected objects and free memory from the	*/
+/* Unselect all the selected elements and free memory from the	*/
 /* selection list.						*/
 /*--------------------------------------------------------------*/
 
@@ -887,7 +780,7 @@ void unselect_all()
 }
 
 /*----------------------------------------------------------------------*/
-/* Select the nearest polygon, searching the hierarchy if necessary.	*/
+/* Select the nearest element, searching the hierarchy if necessary.	*/
 /* Return an pushlist pointer to a linked list containing the 		*/
 /* hierarchy of objects, with the topmost pushlist also containing a	*/
 /* pointer to the polygon found.					*/
@@ -895,13 +788,19 @@ void unselect_all()
 /* the calling routine.							*/
 /*----------------------------------------------------------------------*/
 
-genericptr recurselect(short class, pushlistptr *seltop)
+selection *recurselect(short class, u_char mode, pushlistptr *seltop)
 {
-   genericptr rgen = NULL;
-   short *selectobj;
+   selection *rselect, *rcheck, *lastselect;
+   genericptr rgen;
+   short i, *selectobj, numselect, origselect;
+   short rclass;
    objectptr selobj;
    objinstptr selinst;
    XPoint savesave, tmppt;
+   pushlistptr selnew;
+   short j, unselects;
+   u_char locmode = (mode == MODE_CONNECT) ? UNDO_DONE : mode;
+   u_char recmode = (mode != MODE_CONNECT) ? MODE_RECURSE_WIDE : MODE_RECURSE_NARROW;
 
    if (*seltop == NULL) {
       Fprintf(stderr, "Error: recurselect called with NULL pushlist pointer\n");
@@ -911,49 +810,79 @@ genericptr recurselect(short class, pushlistptr *seltop)
    selinst = (*seltop)->thisinst;
    selobj = selinst->thisobject;
 
-   if ((selectobj = genselectelement(class, UNDO_DONE, selobj, selinst)) != NULL) {
-      rgen = *(selobj->plist + (*selectobj));
-      /* Fprintf(stdout, "%s found in object %s\n", (IS_LABEL(rgen)) ?
-			"label" : "polygon", selobj->name); */
-      draw_normal_selected(selobj, selinst);
-      clearselects();
-   }
-   else if ((selectobj = genselectelement(OBJINST, UNDO_DONE, selobj, selinst)) != NULL) {
-      pushlistptr selnew = (pushlistptr)malloc(sizeof(pushlist));
+   class &= areastruct.filter;		/* apply the selection filter */
 
-      selinst = TOOBJINST(selobj->plist + (*selectobj));	
-      draw_normal_selected(selobj, selinst);
-      clearselects();
+   unselects = 0;
+   rselect = genselectelement(class, locmode, selobj, selinst);
+   if (rselect == NULL) return NULL;
 
-      /* Link hierarchy information to the pushlist linked list */
-      selnew->thisinst = selinst;
-      selnew->next = NULL;
-      (*seltop)->next = selnew;
+   for (i = 0; i < rselect->selects; i++) {
+      rgen = *(selobj->plist + (*(rselect->selectlist + i)));
+      if (rgen->type == OBJINST) {
+         selinst = TOOBJINST(selobj->plist + (*(rselect->selectlist + i)));	
+
+         /* Link hierarchy information to the pushlist linked list */
+         selnew = (pushlistptr)malloc(sizeof(pushlist));
+         selnew->thisinst = selinst;
+         selnew->next = NULL;
+         (*seltop)->next = selnew;
 	 
-      /* Translate areastruct.save into object's own coordinate system */
-      savesave.x = areastruct.save.x;
-      savesave.y = areastruct.save.y;
-      InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
-		selinst->scale, selinst->rotation);
-      areastruct.save.x = tmppt.x;
-      areastruct.save.y = tmppt.y;
-      /* Fprintf(stdout, "objinst %s found in object %s; searching recursively\n",
-		selinst->thisobject->name, selobj->name); */
-      /* Fprintf(stdout, "cursor position originally (%d, %d); "
-		"in new object is (%d, %d)\n",
-		savesave.x, savesave.y,
-		areastruct.save.x, areastruct.save.y); */
-      UPushCTM();
-      UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
-      rgen = recurselect(class, &selnew);
-      UPopCTM();
-      areastruct.save.x = savesave.x;
-      areastruct.save.y = savesave.y;
-   }
-   /* else
-      Fprintf(stdout, "nothing found in object %s\n", selobj->name); */
+         /* Translate areastruct.save into object's own coordinate system */
+         savesave.x = areastruct.save.x;
+         savesave.y = areastruct.save.y;
+         InvTransformPoints(&areastruct.save, &tmppt, 1, selinst->position,
+			selinst->scale, selinst->rotation);
+         areastruct.save.x = tmppt.x;
+         areastruct.save.y = tmppt.y;
+         /* Fprintf(stdout, "objinst %s found in object %s; searching recursively\n",
+			selinst->thisobject->name, selobj->name); */
+         /* Fprintf(stdout, "cursor position originally (%d, %d); "
+			"in new object is (%d, %d)\n",
+			savesave.x, savesave.y,
+			areastruct.save.x, areastruct.save.y); */
+         UPushCTM();
+         UPreMultCTM(DCTM, selinst->position, selinst->scale, selinst->rotation);
+	 rclass = class;
+	 if ((class & (~OBJINST)) == 0) class = ALL_TYPES;
+         rcheck = recurselect(class, recmode, &selnew);
+         UPopCTM();
+         areastruct.save.x = savesave.x;
+         areastruct.save.y = savesave.y;
 
-   return rgen;
+	 /* If rgen is NULL, remove selected object from the list, and	*/
+	 /* remove the last entry from the pushlist stack.		*/
+
+	 if (rcheck == NULL) {
+	    *(rselect->selectlist + i) = -1;
+	    unselects++;
+	    (*seltop)->next = NULL;
+	    if (selnew->next != NULL)
+	       Fprintf(stderr, "Error: pushstack was freed, but was not empty!\n");
+	    free(selnew);
+	 }
+	 else {
+	    for (lastselect = rselect; lastselect->next != NULL; lastselect =
+			lastselect->next);
+	    lastselect->next = rcheck;
+	 }
+      }
+   }
+
+   /* Modify the selection list */
+
+   for (i = 0, j = 0; i < rselect->selects; i++) {
+      if (*(rselect->selectlist + i) >= 0) {
+	 if (i != j)
+	    *(rselect->selectlist + j) = *(rselect->selectlist + i);
+	 j++;
+      }
+   }
+   rselect->selects -= unselects;
+   if (rselect->selects == 0) {
+      freeselection(rselect);
+      rselect = NULL;
+   }
+   return rselect;
 }
 
 /*----------------------------------*/
@@ -1007,4 +936,178 @@ void trackselarea()
    areastruct.save.y = newpos.y;
 }
 
-/*-------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------*/
+/* Polygon distance comparison function for qsort			*/
+/*----------------------------------------------------------------------*/
+
+int dcompare(const void *a, const void *b)
+{
+   XPoint cpt;
+   genericptr agen, bgen;
+   short j, k, adist, bdist;
+
+   cpt.x = areastruct.save.x;
+   cpt.y = areastruct.save.y;
+
+   j = *((short *)a);
+   k = *((short *)b);
+
+   agen = *(topobject->plist + j);
+   bgen = *(topobject->plist + k);
+
+   if (agen->type != POLYGON || bgen->type != POLYGON) return 0;
+
+   adist = closedistance((polyptr)agen, &cpt);
+   bdist = closedistance((polyptr)bgen, &cpt);
+
+   if (adist == bdist) return 0;
+   return (adist < bdist) ? 1 : -1;
+}
+
+/*----------------------------------------------------------------------*/
+/* Compare two selection linked lists					*/
+/*----------------------------------------------------------------------*/
+
+Boolean compareselection(selection *sa, selection *sb)
+{
+   int i, j, match;
+   short n1, n2;
+
+   if ((sa == NULL) || (sb == NULL)) return False;
+   if (sa->selects != sb->selects) return False;
+   match = 0;
+   for (i = 0; i < sa->selects; i++) {
+      n1 = *(sa->selectlist + i);
+      for (j = 0; j < sb->selects; j++) {
+         n2 = *(sb->selectlist + j);
+	 if (n1 == n2) {
+	    match++;
+	    break;
+	 }
+      }
+   }
+   return (match == sa->selects) ? True : False;
+}
+
+/*----------------------------------------------------------------------*/
+/* Recursive selection mechanism					*/
+/*----------------------------------------------------------------------*/
+
+short *recurse_select_element(short class, u_char mode) {
+   pushlistptr seltop, nextptr;
+   selection *rselect;
+   short *newselect, *desel, localpick;
+   static short pick = 0;
+   static selection *saveselect = NULL;
+   int i, j, k, ilast, jlast;
+   Boolean unselect = False;
+
+   seltop = (pushlistptr)malloc(sizeof(pushlist));
+   seltop->thisinst = areastruct.topinstance;
+   seltop->next = NULL;
+
+   /* Definition for unselecting an element */
+
+   if (class < 0) {
+      unselect = True;
+      class = -class;
+   }
+   rselect = recurselect(class, mode, &seltop);
+
+   if (rselect) {
+      /* Order polygons according to nearest point distance. */
+      qsort((void *)rselect->selectlist, (size_t)rselect->selects,
+		sizeof(short), dcompare);
+
+      if (compareselection(rselect, saveselect))
+	 pick++;
+      else
+	 pick = 0;
+
+      localpick = pick % rselect->selects;
+   }
+
+   /* Mechanism for unselecting elements under the cursor	*/
+   /* (Unselect all picked objects)				*/
+
+   if (rselect && unselect) {
+
+      ilast = -1;
+      k = 0;
+      for (i = 0; i < rselect->selects; i++) {
+	 for (j = 0; j < areastruct.selects; j++) {
+	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i)) {
+	       jlast = j;
+	       ilast = i;
+	       if (++k == localpick)
+	          break;
+	    }
+	 }
+	 if (j < areastruct.selects) break;
+      }
+      if (ilast >= 0) {
+	 newselect = rselect->selectlist + ilast;
+         XSetFunction(dpy, areastruct.gc, GXcopy);
+         XTopSetForeground(GSELTOCOLOR(topobject, newselect));
+         geneasydraw(*newselect, DEFAULTCOLOR, topobject, areastruct.topinstance);
+         areastruct.selects--;
+         for (k = jlast; k < areastruct.selects; k++)
+	    *(areastruct.selectlist + k) = *(areastruct.selectlist + k + 1);
+
+         if (areastruct.selects == 0) freeselects();
+
+         /* Register the selection as an undo event */
+         register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+      }
+   }
+
+   else if (rselect) {
+
+      /* Mechanism for selecting objects:			*/
+      /* Count all elements from rselect that are part of	*/
+      /* the current selection.  Pick the "pick"th item (modulo	*/
+      /* total number of items).				*/
+
+      ilast = -1;
+      k = 0;
+      for (i = 0; i < rselect->selects; i++) {
+	 for (j = 0; j < areastruct.selects; j++) {
+	    if (*(areastruct.selectlist + j) == *(rselect->selectlist + i))
+	       break;
+	 }
+	 if (j == areastruct.selects) {
+	    ilast = i;
+	    if (++k == localpick)
+	       break;
+	 }
+      }
+
+      if (ilast >= 0) {
+         newselect = allocselect();
+         *newselect = *(rselect->selectlist + ilast);
+         gendrawselected(newselect, topobject, areastruct.topinstance);
+         setoptionmenu();
+         u2u_snap(&areastruct.save);
+
+         /* Register the selection as an undo event	*/
+	 /* (only if selection changed)			*/
+
+         register_for_undo(XCF_Select, mode, areastruct.topinstance,
+		areastruct.selectlist, areastruct.selects);
+      }
+   }
+
+   /* Cleanup */
+
+   while (seltop != NULL) {
+      nextptr = seltop->next;
+      free(seltop);
+      seltop = nextptr;
+   }
+
+   freeselection(saveselect);
+   saveselect = rselect;
+
+   return areastruct.selectlist;
+}
