@@ -43,11 +43,27 @@
 /* Local structure definitions for holding undo information		*/
 /*----------------------------------------------------------------------*/
 
+typedef struct {
+   float angle1;
+   float angle2;
+   short radius;
+   short yaxis;
+} arcinfo;
+
 typedef struct {
    short number;
    genericptr *element;
    short *idx;
-} selection;
+} uselection;
+
+typedef struct {
+   genericptr element;		/* element being edited */
+   union {
+      stringpart *string;	/* original contents of string, for label */
+      pointlist points;		/* original polygon */
+      arcinfo *arcspecs;	/* original arc values */
+   } save;
+} editelement;
 
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
@@ -60,17 +76,17 @@ extern short eventmode;
 /*----------------------------------------------------------------------*/
 /* remember_selection ---						*/
 /*									*/
-/*   Copy a selection list into a "selection" record.  The selection	*/
+/*   Copy a selection list into a "uselection" record.  The uselection	*/
 /*   record maintains the order of the elements as well as pointers to	*/
 /*   each element, so the original element ordering can be recovered.	*/
 /*----------------------------------------------------------------------*/
 
-selection *remember_selection(objinstptr topinst, short *slist, int number)
+uselection *remember_selection(objinstptr topinst, short *slist, int number)
 {
    int i, idx;
-   selection *newlist;
+   uselection *newlist;
 
-   newlist = (selection *)malloc(sizeof(selection));
+   newlist = (uselection *)malloc(sizeof(uselection));
    if (number > 0) {
       newlist->element = (genericptr *)malloc(number * sizeof(genericptr));
       newlist->idx = (short *)malloc(number * sizeof(short));
@@ -89,11 +105,11 @@ selection *remember_selection(objinstptr topinst, short *slist, int number)
 }
 
 /*----------------------------------------------------------------------*/
-/* Create a selection list in areastruct from the saved selection	*/
+/* Create a selection list in areastruct from the saved uselection	*/
 /* record.								*/
 /*----------------------------------------------------------------------*/
 
-short *regen_selection(objinstptr thisinst, selection *srec)
+short *regen_selection(objinstptr thisinst, uselection *srec)
 {
    int i, j, k;
    genericptr egen;
@@ -141,13 +157,35 @@ short *regen_selection(objinstptr thisinst, selection *srec)
       return slist;
 }
 
+/*----------------------------------------------------------------------*/
+/* free_editelement ---							*/
+/*									*/
+/*   Free memory allocated to an undo record edit element structure.	*/
+/*----------------------------------------------------------------------*/
+
+void free_editelement(editelement *erec)
+{
+   switch (erec->element->type) {
+      case LABEL:
+	 freelabel(erec->save.string);
+         break;
+      case POLYGON: case SPLINE:
+	 free(erec->save.points);
+	 break;
+      case ARC:
+	 free(erec->save.arcspecs);
+	 break;
+   }
+   free(erec);
+}
+
 /*----------------------------------------------------------------------*/
 /* free_selection ---							*/
 /*									*/
 /*   Free memory allocated to an undo record selection list.		*/
 /*----------------------------------------------------------------------*/
 
-void free_selection(selection *selrec)
+void free_selection(uselection *selrec)
 {
    if (selrec->number > 0) {
       free(selrec->element);
@@ -157,28 +195,31 @@ void free_selection(selection *selrec)
 }
 
 /*----------------------------------------------------------------------*/
-/* add_to_selectlist ---						*/
+/* get_original_string ---						*/
 /*									*/
-/*   Add an element to the selection list areastruct.selectlist.	*/
+/* Find the original version of the given label.			*/
 /*----------------------------------------------------------------------*/
 
-void add_to_selectlist(genericptr thisinst)
+stringpart *get_original_string(labelptr thislab)
 {
-   int i;
-   short *sptr;
-   genericptr egen;
+   Undoptr chkrecord, thisrecord;
+   editelement *erec;
+   labelptr elab;
 
-   for (i = 0; i < topobject->parts; i++) {
-      egen = *(topobject->plist + i);
-      if (egen == thisinst)
-	 break;
-   }
-   if (i == topobject->parts) {
-      Fprintf(stderr, "Warning:  Element is not in object!\n");
-   }
-   else {
-      sptr = allocselect();
-      *sptr = (short)i;
+   thisrecord = xobjs.undostack;
+   if (thisrecord == NULL) return NULL;
+
+   for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
+      switch (chkrecord->type) {
+	 case XCF_Edit:
+	    erec = (editelement *)(chkrecord->undodata);
+	    elab = (labelptr)(erec->element);
+	    if (elab != thislab) return NULL;
+	    return erec->save.string;
+	    
+	 default:
+	    return NULL;
+      }
    }
 }
 
@@ -192,7 +233,7 @@ void add_to_selectlist(genericptr thisinst)
 int select_previous(Undoptr thisrecord)
 {
    Undoptr chkrecord;
-   selection *srec;
+   uselection *srec;
 
    unselect_all();
    for (chkrecord = thisrecord->next; chkrecord != NULL; chkrecord = chkrecord->next) {
@@ -208,7 +249,7 @@ int select_previous(Undoptr thisrecord)
 	    return 0;
 	 case XCF_Copy:
 	 case XCF_Select:
-	    srec = (selection *)chkrecord->undodata;
+	    srec = (uselection *)chkrecord->undodata;
 	    areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
 	    areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
 	    return 0;
@@ -254,8 +295,9 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    objectptr delobj;
    objinstptr newinst;
    Undoptr newrecord;
-   selection *srec;
-   genericptr egen;
+   uselection *srec;
+   editelement *erec;
+   genericptr egen, newgen;
    XPoint *fpoint;
 
    /* Do not register new events while running undo/redo actions! */
@@ -355,6 +397,41 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 newrecord->undodata = (char *)egen;
 	 break;
 
+      case XCF_Edit:
+	 /* 1 arg:							*/
+	 /*	egen = element to be edited (genericptr)		*/
+	 egen = va_arg(args, genericptr);
+
+	 /* Create a copy of the element to save */
+	 erec = (editelement *)malloc(sizeof(editelement));
+	 erec->element = egen;
+	 switch(egen->type) {
+	    case LABEL:
+	       erec->save.string =
+			stringcopyall(((labelptr)egen)->string,
+			areastruct.topinstance);
+	       newrecord->idata = ((labelptr)egen)->justify;
+	       break;
+	    case POLYGON:
+	       newrecord->idata = ((polyptr)egen)->number;
+	       erec->save.points = copypoints(((polyptr)egen)->points,
+			newrecord->idata);
+	       break;
+	    case SPLINE:
+	       erec->save.points =
+			copypoints((pointlist)((splineptr)egen)->ctrl, 4);
+	       break;
+	    case ARC:
+	       erec->save.arcspecs = (arcinfo *)malloc(sizeof(arcinfo));
+	       erec->save.arcspecs->angle1 = ((arcptr)egen)->angle1;
+	       erec->save.arcspecs->angle2 = ((arcptr)egen)->angle2;
+	       erec->save.arcspecs->radius = ((arcptr)egen)->radius;
+	       erec->save.arcspecs->yaxis = ((arcptr)egen)->yaxis;
+	       break;
+	 }
+	 newrecord->undodata = (char *)erec;
+	 break;
+
       case XCF_Flip_X: case XCF_Flip_Y:
 	 /* 1 arg:							*/
 	 /*	fpoint = point of flip (XPoint *)			*/
@@ -403,7 +480,8 @@ short undo_one_action()
    Undoptr thisrecord, chkrecord;
    objectptr thisobj;
    objinstptr thisinst;
-   selection *srec;
+   uselection *srec;
+   editelement *erec;
    short *slist;
    XPoint *delta;
    int i, snum, npage, opage;
@@ -416,6 +494,7 @@ short undo_one_action()
       Fprintf(stderr, "Nothing to undo!\n");
       return;
    }
+
    xobjs.undostack = thisrecord->next;
    xobjs.redostack = thisrecord;
 
@@ -504,8 +583,67 @@ short undo_one_action()
 	 }
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 switch (erec->element->type) {
+	    case LABEL: {
+	       stringpart *tmpstr;
+	       int tmpjust;
+	       labelptr elab = (labelptr)(erec->element);
+	       undrawtext(elab);
+	       tmpstr = elab->string;
+	       tmpjust = (int)elab->justify;
+	       elab->string = stringcopyback(erec->save.string,
+				thisrecord->thisinst);
+	       elab->justify = (short)thisrecord->idata;
+	       erec->save.string = tmpstr;
+	       thisrecord->idata = tmpjust;
+	       resolveparams(thisrecord->thisinst);
+	       redrawtext(elab);
+	    } break;
+	    case POLYGON: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly = (polyptr)(erec->element);
+	       tmppts = epoly->points;
+	       tmpnum = epoly->number;
+	       epoly->points = erec->save.points;
+	       epoly->number = thisrecord->idata;
+	       erec->save.points = tmppts;
+	       thisrecord->idata = tmpnum;
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case SPLINE: {
+	       pointlist tmppts;
+	       splineptr espline = (splineptr)(erec->element);
+	       tmppts = copypoints((pointlist)espline->ctrl, 4);
+	       for (i = 0; i < 4; i++)
+		   espline->ctrl[i] = *(erec->save.points + i);
+	       free(erec->save.points);
+	       erec->save.points = tmppts;
+	       calcspline(espline);
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areastruct.area, NULL, NULL);
+	    }
+	 }
+	 break;
+
       case XCF_Library_Pop:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 			srec->number, DRAW);
@@ -514,7 +652,7 @@ short undo_one_action()
 	 break;
 
       case XCF_Copy:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 			srec->number, DRAW);
@@ -569,9 +707,9 @@ short undo_one_action()
 
 void undo_action()
 {
-   short idx = undo_one_action();
+   short idx = undo_one_action(False);
    while (xobjs.undostack && xobjs.undostack->idx == idx)
-      undo_one_action();
+      undo_one_action(False);
 }
 
 /*----------------------------------------------------------------------*/
@@ -586,8 +724,9 @@ short redo_one_action()
    genericptr egen;
    short *slist;
    XPoint *delta;
-   selection *srec;
-   int tpage, snum;
+   uselection *srec;
+   editelement *erec;
+   int i, tpage, snum;
 
    /* Undo the recorded action and shift the undo record pointer.	*/
 
@@ -605,7 +744,7 @@ short redo_one_action()
 
    switch(thisrecord->type) {
       case XCF_Delete:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 thisobj = delete_element(thisrecord->thisinst, slist,
 		srec->number, DRAW);
@@ -618,7 +757,7 @@ short redo_one_action()
 
       /* Unfinished! */
       case XCF_Select_Save:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 slist = regen_selection(thisrecord->thisinst, srec);
 	 break;
 
@@ -636,7 +775,7 @@ short redo_one_action()
 
       case XCF_Select:
 	 unselect_all();
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 areastruct.selectlist = regen_selection(thisrecord->thisinst, srec);
 	 areastruct.selects = (areastruct.selectlist) ? srec->number : 0;
 	 draw_all_selected();
@@ -674,6 +813,65 @@ short redo_one_action()
 	 drawarea(areastruct.area, NULL, NULL);
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 switch (erec->element->type) {
+	    case LABEL: {
+	       stringpart *tmpstr;
+	       int tmpjust;
+	       labelptr elab = (labelptr)(erec->element);
+	       undrawtext(elab);
+	       tmpstr = elab->string;
+	       tmpjust = (int)elab->justify;
+	       elab->string = stringcopyback(erec->save.string,
+				thisrecord->thisinst);
+	       elab->justify = (short)thisrecord->idata;
+	       erec->save.string = tmpstr;
+	       thisrecord->idata = tmpjust;
+	       resolveparams(thisrecord->thisinst);
+	       redrawtext(elab);
+	    } break;
+	    case POLYGON: {
+	       pointlist tmppts;
+	       int tmpnum;
+	       polyptr epoly = (polyptr)(erec->element);
+	       tmppts = epoly->points;
+	       tmpnum = epoly->number;
+	       epoly->points = erec->save.points;
+	       epoly->number = thisrecord->idata;
+	       erec->save.points = tmppts;
+	       thisrecord->idata = tmpnum;
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case SPLINE: {
+	       pointlist tmppts;
+	       splineptr espline = (splineptr)(erec->element);
+	       tmppts = copypoints((pointlist)espline->ctrl, 4);
+	       for (i = 0; i < 4; i++)
+		   espline->ctrl[i] = *(erec->save.points + i);
+	       free(erec->save.points);
+	       erec->save.points = tmppts;
+	       calcspline(espline);
+	       drawarea(areastruct.area, NULL, NULL);
+	    } break;
+	    case ARC: {
+	       arcinfo tmpinfo;
+	       arcptr earc = (arcptr)(erec->element);
+	       tmpinfo.angle1 = earc->angle1; 
+	       tmpinfo.angle2 = earc->angle2; 
+	       tmpinfo.radius = earc->radius; 
+	       tmpinfo.yaxis = earc->yaxis; 
+	       earc->angle1 = erec->save.arcspecs->angle1;
+	       earc->angle2 = erec->save.arcspecs->angle2;
+	       earc->radius = erec->save.arcspecs->radius;
+	       earc->yaxis = erec->save.arcspecs->yaxis;
+	       *(erec->save.arcspecs) = tmpinfo;
+	       calcarc(earc);
+	       drawarea(areastruct.area, NULL, NULL);
+	    }
+	 }
+	 break;
+
       case XCF_Flip_X:
 	 areastruct.save = *((XPoint *)thisrecord->undodata);
 	 elementflip();
@@ -779,7 +977,8 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 {
    u_int type;
    objectptr uobj;
-   selection *srec;
+   uselection *srec;
+   editelement *erec;
 
    type = thisrecord->type;
    switch (type) {
@@ -789,7 +988,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	    reset(uobj, DESTROY);
 	 }
 	 else { /* MODE_REDO */
-	    srec = (selection *)thisrecord->undodata;
+	    srec = (uselection *)thisrecord->undodata;
 	    free_selection(srec);
 	 }
 	 break;
@@ -802,10 +1001,15 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	    free(thisrecord->undodata);
 	 break;
 
+      case XCF_Edit:
+	 erec = (editelement *)thisrecord->undodata;
+	 free_editelement(erec);
+	 break;
+
       case XCF_Copy:
       case XCF_Library_Pop:
 	 if (mode == MODE_UNDO) {
-	    srec = (selection *)thisrecord->undodata;
+	    srec = (uselection *)thisrecord->undodata;
 	    free_selection(srec);
 	 }
 	 else { /* MODE_REDO */
@@ -819,7 +1023,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 	 break;
 
       case XCF_Select:
-	 srec = (selection *)thisrecord->undodata;
+	 srec = (uselection *)thisrecord->undodata;
 	 free_selection(srec);
 	 break;
 
